searchData={"content_type":"text/markdown","items":[{"doc":"An `ExBuffer` is a process that maintains a collection of items and flushes\nthem once certain conditions have been met.\n\nAn `ExBuffer` can flush based on a timeout, a maximum length (item count), a\nmaximum byte size, or a combination of the three. When multiple conditions are\nused, the `ExBuffer` will flush when the **first** condition is met.\n\n`ExBuffer` also includes a number of helpful tools for testing and debugging.","ref":"ExBuffer.html","title":"ExBuffer","type":"behaviour"},{"doc":"Lazily chunks an enumerable based on `ExBuffer` flush conditions.\n\nThis function currently supports length and size conditions. If multiple\nconditions are specified, a chunk is emitted once the **first** condition is\nmet (just like an `ExBuffer` process).\n\nWhile this function is useful in it's own right, it's included primarily as\nanother way to synchronously test applications that use `ExBuffer`.","ref":"ExBuffer.html#chunk/2","title":"ExBuffer.chunk/2","type":"function"},{"doc":"An enumerable can be chunked with the following options:\n\n  * `:max_length` - A non-negative integer representing the maximum allowed\n    length (item count) of a chunk. By default, there is no max length. (Optional)\n\n  * `:max_size` - A non-negative integer representing the maximum allowed size\n    (in bytes) of a chunk. The `:size_callback` option determines how item size\n    is computed. By default, there is no max size. (Optional)\n\n  * `:size_callback` - The function that will be invoked to deterime the size\n    of an item. For more information, see `ExBuffer.start_link/2`. (Optional)\n\n> ##","ref":"ExBuffer.html#chunk/2-options","title":"Options - ExBuffer.chunk/2","type":"function"},{"doc":">\n> Including neither `:max_length` nor `:max_size` is permitted but will result in a\n> single chunk being emitted. One can achieve a similar result in a more performant\n> way using `Stream.into/2`. In that same vein, including only a `:max_length`\n> condition makes this function a less performant version of `Stream.chunk_every/2`.\n> This function is optimized for chunking by either size or size **and** count. Any other\n> chunking strategy can likely be achieved in a more efficient way using other methods.","ref":"ExBuffer.html#chunk/2-warning-warning","title":"Warning {: .warning} - ExBuffer.chunk/2","type":"function"},{"doc":"iex> enum = [\"foo\", \"bar\", \"baz\", \"foobar\", \"barbaz\", \"foobarbaz\"]\n    ...> {:ok, enum} = ExBuffer.chunk(enum, max_length: 3, max_size: 10)\n    ...> Enum.into(enum, [])\n    [[\"foo\", \"bar\", \"baz\"], [\"foobar\", \"barbaz\"], [\"foobarbaz\"]]\n\n    iex> enum = [\"foo\", \"bar\", \"baz\"]\n    ...> {:ok, enum} = ExBuffer.chunk(enum, max_size: 8, size_callback: &(byte_size(&1) + 1))\n    ...> Enum.into(enum, [])\n    [[\"foo\", \"bar\"], [\"baz\"]]\n\n    iex> ExBuffer.chunk([\"foo\", \"bar\", \"baz\"], max_length: -5)\n    {:error, :invalid_limit}","ref":"ExBuffer.html#chunk/2-examples","title":"Examples - ExBuffer.chunk/2","type":"function"},{"doc":"Lazily chunks an enumerable based on `ExBuffer` flush conditions and raises an `ArgumentError`\nwith invalid options.\n\nFor more information on this function's usage, purpose, and options, see `ExBuffer.chunk!/2`.","ref":"ExBuffer.html#chunk!/2","title":"ExBuffer.chunk!/2","type":"function"},{"doc":"iex> [\"foo\", \"bar\", \"baz\", \"foobar\", \"barbaz\", \"foobarbaz\"]\n    ...> |> ExBuffer.chunk!(max_length: 3, max_size: 10)\n    ...> |> Enum.into([])\n    [[\"foo\", \"bar\", \"baz\"], [\"foobar\", \"barbaz\"], [\"foobarbaz\"]]\n\n    iex> [\"foo\", \"bar\", \"baz\"]\n    ...> |> ExBuffer.chunk!(max_size: 8, size_callback: &(byte_size(&1) + 1))\n    ...> |> Enum.into([])\n    [[\"foo\", \"bar\"], [\"baz\"]]\n\n    iex> ExBuffer.chunk!([\"foo\", \"bar\", \"baz\"], max_length: -5)\n    ** (ArgumentError) invalid limit","ref":"ExBuffer.html#chunk!/2-examples","title":"Examples - ExBuffer.chunk!/2","type":"function"},{"doc":"Dumps the contents of the given `ExBuffer` to a list, bypassing a flush\ncallback and resetting the buffer.\n\nWhile this functionality may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#dump/1","title":"ExBuffer.dump/1","type":"function"},{"doc":"iex> ExBuffer.insert(:buffer, \"foo\")\n    iex> ExBuffer.insert(:buffer, \"bar\")\n    iex> ExBuffer.dump(:buffer)\n    [\"foo\", \"bar\"]","ref":"ExBuffer.html#dump/1-example","title":"Example - ExBuffer.dump/1","type":"function"},{"doc":"Flushes the given `ExBuffer`, regardless of whether or not the flush conditions\nhave been met.\n\nWhile this functionality may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#flush/2","title":"ExBuffer.flush/2","type":"function"},{"doc":"An `ExBuffer` can be flushed with the following options:\n\n  * `:async` - A boolean representing whether or not the flush will be asynchronous.\n    By default, this value is `true`. (Optional)","ref":"ExBuffer.html#flush/2-options","title":"Options - ExBuffer.flush/2","type":"function"},{"doc":"iex> ExBuffer.insert(:buffer, \"foo\")\n    iex> ExBuffer.insert(:buffer, \"bar\")\n    ...>\n    ...> # Invokes callback on [\"foo\", \"bar\"]\n    iex> ExBuffer.flush(:buffer)\n    :ok","ref":"ExBuffer.html#flush/2-example","title":"Example - ExBuffer.flush/2","type":"function"},{"doc":"Invoked to flush an `ExBuffer`.\n\nThe first argument (`data`) is a list of items inserted into the `ExBuffer` and the\nsecond argument (`opts`) is a keyword list of flush options. See the `:flush_callback`\nand `:flush_meta` options for `ExBuffer.start_link/2` for more information.\n\nThis callback can return any term as the return value is disregarded by the `ExBuffer`.\n\nThis callback is required.","ref":"ExBuffer.html#c:handle_flush/2","title":"ExBuffer.handle_flush/2","type":"callback"},{"doc":"Invoked to determine the size of an inserted item.\n\nThe only argument (`item`) is any term that was inserted into the `ExBuffer`.\n\nThis callback must return a non-negative integer representing the item's byte size.\n\nThis callback is optional. See the `:size_callback` option for `ExBuffer.start_link/2`\nfor information about the default implementation.","ref":"ExBuffer.html#c:handle_size/1","title":"ExBuffer.handle_size/1","type":"callback"},{"doc":"Inserts the given item into the given `ExBuffer`.","ref":"ExBuffer.html#insert/2","title":"ExBuffer.insert/2","type":"function"},{"doc":"iex> ExBuffer.insert(:buffer, \"foo\")\n    :ok","ref":"ExBuffer.html#insert/2-example","title":"Example - ExBuffer.insert/2","type":"function"},{"doc":"Returns the length (item count) of the given `ExBuffer`.\n\nWhile this functionality may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#length/1","title":"ExBuffer.length/1","type":"function"},{"doc":"iex> ExBuffer.insert(:buffer, \"foo\")\n    iex> ExBuffer.insert(:buffer, \"bar\")\n    iex> ExBuffer.length(:buffer)\n    2","ref":"ExBuffer.html#length/1-example","title":"Example - ExBuffer.length/1","type":"function"},{"doc":"Returns the time (in ms) before the next scheduled flush of the given `ExBuffer`.\n\nIf the given `ExBuffer` does not have a timeout, this function returns `nil`.\n\nWhile this functionality may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#next_flush/1","title":"ExBuffer.next_flush/1","type":"function"},{"doc":"iex> next_flush = ExBuffer.next_flush(:buffer)\n    ...>\n    ...> # Assuming :buffer has a timeout...\n    iex> is_integer(next_flush)\n    true","ref":"ExBuffer.html#next_flush/1-example","title":"Example - ExBuffer.next_flush/1","type":"function"},{"doc":"Retuns the size (in bytes) of the given `ExBuffer`.\n\nFor more information on how item size is computed, see `ExBuffer.start_link/2`.\n\nWhile this functionality may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#size/1","title":"ExBuffer.size/1","type":"function"},{"doc":"iex> ExBuffer.insert(:buffer, \"foo\")\n    iex> ExBuffer.insert(:buffer, \"bar\")\n    iex> ExBuffer.size(:buffer)\n    6","ref":"ExBuffer.html#size/1-example","title":"Example - ExBuffer.size/1","type":"function"},{"doc":"Starts an `ExBuffer` process linked to the current process.\n\nThe first argument argument (`module`) is optional. It is intended to be used when\ncalling this function from a module that implements the `ExBuffer` behaviour. When\na module is passed, it may interact with the options that were passed in:\n\n* If the module implements the `handle_flush/2` callback, it will override the\n  `:flush_callback` option.\n\n* If the module implements the `handle_size/1` callback, it will override the\n  `:size_callback` option.\n\n* If a `:name` option is not present, the module name will be used.","ref":"ExBuffer.html#start_link/2","title":"ExBuffer.start_link/2","type":"function"},{"doc":"An `ExBuffer` can be started with the following options:\n\n  * `:flush_callback` - The function that will be invoked to handle a flush.\n    This function should expect two parameters: a list of items and a keyword\n    list of flush opts. The flush opts include the size and length of the buffer\n    at the time of the flush and optionally include any provided metadata (see\n    `:flush_meta` for more information). This function can return any term as the\n    return value is not used by the `ExBuffer`. (Required)\n\n  * `:buffer_timeout` - A non-negative integer representing the maximum time\n    (in ms) allowed between flushes of the `ExBuffer`. Once this amount of time\n    has passed, the `ExBuffer` will be flushed. By default, an `ExBuffer` does not\n    have a timeout. (Optional)\n\n  * `:flush_meta` - A term to be included in the flush opts under the `meta` key.\n    By default, this value will be `nil`. (Optional)\n\n  * `:max_length` - A non-negative integer representing the maximum allowed\n    length (item count) of the `ExBuffer`. Once the limit is hit, the `ExBuffer` will\n    be flushed. By default, an `ExBuffer` does not have a max length. (Optional)\n\n  * `:max_size` - A non-negative integer representing the maximum allowed size\n    (in bytes) of the `ExBuffer`. Once the limit is hit (or exceeded), the `ExBuffer`\n    will be flushed. The `:size_callback` option determines how item size is\n    computed. By default, an `ExBuffer` does not have a max size. (Optional)\n\n  * `:size_callback` - The function that will be invoked to determine the size\n    of an item. This function should expect a single parameter representing an\n    item and should return a single non-negative integer representing that item's\n    byte size. The default `ExBuffer` size callback is `Kernel.byte_size/1`\n    (`:erlang.term_to_binary/1` is used to convert non-bitstring inputs to binary\n    if necessary). (Optional)\n\nAdditionally, an ExBuffer can also be started with any `GenServer` options.","ref":"ExBuffer.html#start_link/2-options","title":"Options - ExBuffer.start_link/2","type":"function"},{"doc":"Errors returned by `ExBuffer` functions.","ref":"ExBuffer.html#t:error/0","title":"ExBuffer.error/0","type":"type"},{"doc":"# ExBuffer\n\n![CI](https://github.com/gdwoolbert3/ex_buffer/actions/workflows/ci.yml/badge.svg)\n[![Package](https://img.shields.io/hexpm/v/ex_buffer.svg)](https://hex.pm/packages/ex_buffer)\n\nAn `ExBuffer` is a process that maintains a collection of items and flushes them once certain conditions have\nbeen met.","ref":"readme.html","title":"ExBuffer","type":"extras"},{"doc":"This package can be installed by adding `:ex_buffer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_buffer \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - ExBuffer","type":"extras"},{"doc":"For additional documentation, see [HexDocs](https://hexdocs.pm/ex_buffer/readme.html).","ref":"readme.html#documentation","title":"Documentation - ExBuffer","type":"extras"},{"doc":"We can easily start an `ExBuffer` by adding it directly to a supervision tree.\n\n```elixir\nopts = [\n  flush_callback: fn data, _ -> IO.inspect(data) end,\n  max_length: 3,\n  name: :buffer\n]\n\nchildren = [\n  {ExBuffer, opts}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nOnce the buffer has been started, we can insert items.\n\n```elixir\nExBuffer.insert(:buffer, \"foo\")\nExBuffer.insert(:buffer, \"bar\")\n```\n\nAnd, once any of the configured conditions have been met, the buffer will automatically flush.\n\n```elixir\nExBuffer.insert(:buffer, \"baz\")\n# ExBuffer flushes asynchronously and outputs [\"foo\", \"bar\", \"baz\"]\n```","ref":"readme.html#getting-started","title":"Getting Started - ExBuffer","type":"extras"},{"doc":"`ExBuffer` is designed to be highly customizable, allowing it to be used in any number of scenarios.\n\n#","ref":"readme.html#examples","title":"Examples - ExBuffer","type":"extras"},{"doc":"For example, we can use the `ExBuffer` behaviour to create a buffer with both a size limit and a time limit.\n\n```elixir\ndefmodule Buffer do\n  use ExBuffer\n\n  def start_link(opts \\\\ []) do\n    opts = Keyword.merge([max_size: 8, buffer_timeout: 30_000], opts)\n    ExBuffer.start_link(__MODULE__, opts)\n  end\n\n  def insert(item) do\n    ExBuffer.insert(__MODULE__, item)\n  end\n\n  @impl ExBuffer\n  def handle_flush(data, _opts) do\n    IO.inspect(data)\n  end\n\n  @impl ExBuffer\n  def handle_size(item) do\n    byte_size(item) + 1\n  end\nend\n```\n\nWe can easily start the `Buffer` process from above to see it in action.\n\n```elixir\nBuffer.start_link()\n\nBuffer.insert(\"foo\")\nBuffer.insert(\"bar\")\n# Buffer flushes asynchronously and outputs [\"foo\", \"bar\"]\n\nBuffer.insert(\"baz\")\n# After 30 seconds pass...\n# Buffer flushes asynchronously and outputs [\"baz\"]\n```\n\n#","ref":"readme.html#simple-buffer","title":"Simple Buffer - ExBuffer","type":"extras"},{"doc":"Alternatively, we could also use `ExBuffer` in conjunction with Elixir's `PartitionSupervisor` to easily\ncreate a partitioned buffer with dynamic flush behavior.\n\n```elixir\ndefmodule PartitionedBuffer do\n  use Supervisor\n\n  def start_link(opts \\\\ []) do\n    opts = Keyword.merge([max_length: 3], opts)\n    Supervisor.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  def insert(item, partition) do\n    ExBuffer.insert({:via, PartitionSupervisor, {:buffer, partition}}, item)\n  end\n\n  @impl Supervisor\n  def init(opts) do\n    part_sup_opts = [\n      name: :buffer,\n      child_spec: {ExBuffer, opts},\n      partitions: 2,\n      with_arguments: fn [opts], part ->\n        flush_callback = fn data, _ -> handle_flush(data, part) end\n        [Keyword.put(opts, :flush_callback, flush_callback)]\n      end\n    ]\n\n    children = [\n      {PartitionSupervisor, part_sup_opts}\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\n\n  defp handle_flush(data, partition) do\n    IO.inspect({partition, data})\n  end\nend\n```\n\nWe can easily start the `PartitionedBuffer` process from above to see it in action.\n\n```elixir\nPartitionedBuffer.start_link()\n\nPartitionedBuffer.insert(\"foo\", 0)\nPartitionedBuffer.insert(\"foo\", 1)\nPartitionedBuffer.insert(\"bar\", 0)\nPartitionedBuffer.insert(\"bar\", 1)\nPartitionedBuffer.insert(\"baz\", 0)\n# Partition 0 flushes asynchronously and outputs {0, [\"foo\", \"bar\", \"baz\"]}\n\nPartitionedBuffer.insert(\"baz\", 1)\n# Partition 1 flushes asynchronously and outputs {1, [\"foo\", \"bar\", \"baz\"]}\n```","ref":"readme.html#partitioned-buffer","title":"Partitioned Buffer - ExBuffer","type":"extras"},{"doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\n\n## [0.3.0] - 2023-10-26\n\n#","ref":"changelog.html","title":"Change Log","type":"extras"},{"doc":"* Added an `ExBuffer` behaviour.\n* Added a non-banged version of `ExBuffer.chunk!/2`.\n\n#","ref":"changelog.html#added","title":"Added - Change Log","type":"extras"},{"doc":"* Split code back into multiple modules to simplify feature addition.\n* Reorganized unit tests.\n\n#","ref":"changelog.html#changed","title":"Changed - Change Log","type":"extras"},{"doc":"* [GitHub](https://github.com/gdwoolbert3/ex_buffer/releases/tag/v0.3.0)\n\n## [0.2.1] - 2023-10-24\n\n#","ref":"changelog.html#links","title":"Links - Change Log","type":"extras"},{"doc":"* Condensed code into a single module.\n\n#","ref":"changelog.html#changed","title":"Changed - Change Log","type":"extras"},{"doc":"* [GitHub](https://github.com/gdwoolbert3/ex_buffer/releases/tag/v0.2.1)\n* [HexDocs](https://hexdocs.pm/ex_buffer/0.2.1/readme.html)\n \n## [0.2.0] - 2023-10-23\n\n#","ref":"changelog.html#links","title":"Links - Change Log","type":"extras"},{"doc":"* Added the ability to provide a custom callback for determining item size (see `:size_callback` option for\n  both `ExBuffer.start_link/1` and `ExBuffer.chunk!/2`).\n* Added the ability to include metadata in the flush callback (see `:flush_meta` option for\n  `ExBuffer.start_link/1`).\n* Added the ability to get the time until the next scheduled flush (see `ExBuffer.next_flush/1`).\n* Added benchee dependency for local benchmarking.\n\n#","ref":"changelog.html#added","title":"Added - Change Log","type":"extras"},{"doc":"* The `:callback` option for `ExBuffer.start_link/1` had been renamed to `:flush_callback` and now expects a\n  function of arity 2 (see `ExBuffer.start_link/1`).\n* Updated function docs to include doctests.\n* Updated README.md to include a more sophisticated use case.\n* Updated README.md to display build status and [Hex](https://hex.pm/) version.\n\n#","ref":"changelog.html#changed","title":"Changed - Change Log","type":"extras"},{"doc":"* [GitHub](https://github.com/gdwoolbert3/ex_buffer/releases/tag/v0.2.0)\n* [HexDocs](https://hexdocs.pm/ex_buffer/0.2.0/readme.html)\n \n## [0.1.0] - 2023-10-20\n \n#","ref":"changelog.html#links","title":"Links - Change Log","type":"extras"},{"doc":"* Initial release.\n\n#","ref":"changelog.html#added","title":"Added - Change Log","type":"extras"},{"doc":"* [HexDocs](https://hexdocs.pm/ex_buffer/0.1.0/readme.html)","ref":"changelog.html#links","title":"Links - Change Log","type":"extras"}]}